#!/usr/bin/env python
#-*- coding: ISO-8859-1 -*-
##########################################################################
#  Copyright (C) 2008 Valentin Kuznetsov <vkuznet@gmail.com>
#  All rights reserved.
#  Distributed under the terms of the BSD License.  The full license is in
#  the file doc/LICENSE, distributed as part of this software.
##########################################################################
"""
DB Shell class and utilities
"""
# system modules we use
import os, sys, string, time, types, traceback, thread, popen2, re

# SQLAlchemy modules
from sqlalchemy import *
import sqlalchemy
from sqlalchemy.databases import information_schema
from sqlalchemy.databases import oracle
from sqlalchemy.databases import mysql
from sqlalchemy.databases import sqlite

# local modules
from dbcmd import *
from dbprint import *
from utils import *
from dbdot import *

class DBManager(object):
  """
     DBSDD class takes care of authentication with underlying DB using
     DBS_DBPARAM. SQLAlchemy to provides DB access layer, currently I added
     SQLite, MySQL, ORACLE support. The tables automatically retrieves from DB
     and internal schema can be reconstructed on the fly.

     For description of SQLAlchemy pleasee see
     http://www.sqlalchemy.org
                                
  """
  def __init__(self,verbose=0):
      """
         DBSDD constructor. 
         @type verbose: boolean or integer
         @param verbose: verbosity level
         @rtype : none
         @return: none
      """
      self.verbose   = verbose
      self.printMgr  = PrintOutput()
      self.printMethod = "txt"
      self.tCache    = []
      self.formats   = ['jpg','png','gif','svg','ps']
      self.members   = ['engine','dbTables','tableNames','dbType','dbOwner','dbSchema','metaDict','drivers','aliases']
      self.clear()

  def clear(self):
      for dict in self.members:
          setattr(self,dict,{})
      self.con       = None
      self.limit     = None
      self.offset    = None
      self.query     = ""
      self.result    = ""
      
  def writeGraph(self,dbAlias,format=None):
      fileName="%s.dot"%dbAlias
      print "Writing graph of DB schema to",fileName
      dot=DotGraph(file(fileName,"w"))
      tDict = self.loadTables(dbAlias) # load all tables before building a graph
      for key in tDict.keys():
          table=tDict[key]
          tableName=key
          foreignKeys=table.foreign_keys
          for fk in foreignKeys:
              right=fk.column.table.name
              if right!='person': # exclude person table
                 dot.AddEdge(tableName,right)
      dot.Finish()
      if  format:
          if  format not in self.formats:
              msg = "Unsupported format '%s', please use %s"%(format,str(self.formats))
              raise msg
          cmd = "dot -v -T%s -O %s"%(format,fileName)
          try:
              status = os.system(cmd)
              print "Executing",cmd
          except:
              print "Fail to execute",cmd
              print "Please verify that you have dot installed on your system"
              print traceback.print_exc()
              pass

  def dbname(self,arg):
      if self.drivers.has_key(arg):
         arg = self.drivers[arg]
      dbType,dbName,dbUser,dbPass,host,port,owner,fileName=self.parse(arg)
      if dbType.lower()=='oracle': 
         name=dbName
         if owner: name+="-%s"%owner
         return "%s-%s |\#> "%(dbType,name)
      if dbType.lower()=='mysql': return "%s-%s-%s |\#> "%(dbType,dbName,host)
      if dbType.lower()=='sqlite': 
         f = fileName.split("/")[-1]
         return "%s-%s |\#> "%(dbType,f)
      return "%s-%s |\#> "%(dbType,dbName)

  def showTable(self,dbAlias):
      if  self.tableNames.has_key(dbAlias):
          tables = self.tableNames[dbAlias]
          tables.sort()
          self.printList(tables,"\nFound tables")
      else:
          self.printList([],"\nFound tables")

  def showDB(self):
      print "Show DB content"
      
  def plot(self,dbAlias,query):
      try:
          result = self.con.execute(query)
      except:
          raise traceback.print_exc()
      xList  = []
      yList  = []
      tList  = []
      for item in result:
          if type(item) is types.StringType:
              raise item+"\n"
          if not tList:
             tList=list(item.keys())
          if len(item)!=2:
             raise "Plot support only 2-dimensional data"
          x,y = item
          xList.append(x)
          yList.append(y)
      thread.start_new_thread(self.pylabPlot,(xList,yList,tList,query))

  def pylabPlot(self,xList,yList,tList,title):
      import pylab
      pylab.plot(xList,yList)
      pylab.xlabel(tList[0])
      pylab.ylabel(tList[1])
      pylab.title(title)
      pylab.grid(True)
      pylab.show()
      
  def desc(self,dbAlias,table):
      tables = self.loadTables(dbAlias,table) # load table from DB
      tabObj = tables[table]
      print tabObj
      tList = ['Name','Type','Key','Default','Autoincrment','Foreign Keys']
      oList = [] # contains tuple of values for tList
      lList = [len(x) for x in tList] # column width list
      for col in tabObj.columns:
          key   = ""
          if col.unique: key="unique"
          elif col.primary_key: key="primary"
          value = "NULL"
          if col.default: value = col.default
          foreignKeys=""
          for fk in col.foreign_keys:
              foreignKeys+="%s "%fk.column
          vList = (col.name,col.type,key,value,col.autoincrement,foreignKeys)
          oList.append(vList)
          for idx in xrange(0,len(vList)):
              if lList[idx]<len(str(vList[idx])): lList[idx]=len(str(vList[idx]))
      oList.sort() 
      self.printTable(tList,oList,lList)

  def dump(self,dbAlias,fileName=None):
      tables = self.loadTables(dbAlias) # load all tables from DB in order to dump DDL
      dbType = self.dbType[dbAlias]
      msg    = "--\n-- Dump %s.\n-- %s\n"%(dbAlias,makeTIME(time.time()))
      if  fileName:
          file= open(fileName,'w') 
          file.write(msg)
      else:
          print msg
      for tName in tables.keys():
          table = tables[tName]
          try:
             table.create()
          except:
             error = sys.exc_info()[1]
             if  fileName:
                 file.write("%s;\n"%error.statement)
             else:
                 print "%s;\n"%error.statement
          try:
              result = self.con.execute("SELECT * FROM %s"%tName)
              for item in result:
                  if type(item) is types.StringType:
                      raise item+"\n"
                  columns = str(item.keys()).replace("[","(").replace("]",")")
                  values  = str(item.values()).replace("[","(").replace("]",")")
                  stm = "INSERT INTO %s %s VALUES %s;"%(tName,columns,values)
                  if  fileName:
                      file.write(stm+"\n")
                  else:
                      print stm
          except:
              raise traceback.print_exc()
      if fileName:
         file.close()

  def migrate(self,dbAlias,arg):
      tables = self.loadTables(dbAlias) # load all tables from DB
      dbCon  = self.con
      self.connect(arg)
      newDBAlias = self.aliases[arg]
      remoteEngine = self.engine[newDBAlias]
      meta = sqlalchemy.MetaData()
      meta.bind = remoteEngine
      for table in tables.keys():
          tables[table].create(bind=remoteEngine)
          newTable = tables[table].tometadata(meta)
          query = "select * from %s"%table
          try:
              result = dbCon.execute(query)
          except:
              raise traceback.print_exc()
          con = remoteEngine.connect()
          for item in result:
              if type(item) is types.StringType:
                  raise item+"\n"
              ins = newTable.insert(values=item.values())
              con.execute(ins)
          con.close()
      print "The content of '%s' has been successfully migrated to '%s'"%(dbAlias,newDBAlias)
      print "You may invoke "+self.printMgr.msgGREEN("connect %s"%arg)+" command now."
      self.close(newDBAlias)

  def createAlias(self,name,params):
      self.dbalias[name]=params

  def page(self,arg):
      iList = arg.split()
      if  len(iList)==2:
          self.offset= int(iList[0])
          self.limit = int(iList[1])
      elif len(iList)==1:
          self.limit = int(iList[0])
          self.offset= 0
      else:
          msg="ERROR: page support only one or two arguments"
          msg+= cmdDictExt['page']+"\n"
          raise msg+"\n"

  def next(self):
      self.offset+=self.limit
      if len(self.tCache)>self.offset:
         result = self.tCache[self.offset:self.offset+self.limit]
      else:
         result = self.result
      self.printResult(result,self.query)

  def prev(self):
      self.offset-=self.limit
      if self.offset<0:
         return
      result = self.tCache[self.offset:self.offset+self.limit]
      self.printResult(result,self.query)

  def execute(self,dbAlias,query,listResults=1):
      self.tCache=[]
      self.query=query
      try:
          result = self.con.execute(query)
          self.result = result
      except:
          raise traceback.print_exc()
      if not listResults: return
      self.printResult(result,query)

  def printResult(self,result,query):
      oList  = []
      tList  = []
      lList  = []
      # NEW
      if self.limit and not (type(result) is types.ListType):
         result = fetchFromTo(result,self.limit,self.offset)
      # END OF NEW
      for item in result:
          if type(item) is types.StringType:
             raise item+"\n"
          if not (type(result) is types.ListType):
             self.tCache.append(item)
          if not tList:
             tList=list(item.keys())
             lList=[len(x) for x in tList]
          vList = item.values()
          oList.append(vList)
          for idx in xrange(0,len(vList)):
              if lList[idx]<len(str(vList[idx])): lList[idx]=len(str(vList[idx]))
      self.printTable(tList,oList,lList,query)

  def dropDB(self,dbAlias):
      tables = self.loadTables(dbAlias)
      for tName in tables.keys():
          table = tables[tName]
          try:
             table.drop()
          except:
             traceback.print_exc()
      self.dbTables.pop(dbAlias)

  def dropTable(self,dbAlias,tableName):
      tables = self.loadTables(dbAlias,tableName)
      if  tables.has_key(tableName):
          tabObj = tables[tableName]
          try:
              tabObj.drop()
          except:
#              traceback.print_exc()
              pass
          try:
              tables.pop(tableName)
          except:
#              traceback.print_exc()
              pass
          try:
              print self.dbTables
              self.dbTables[dbAlias].pop(tableName)
          except:
#              traceback.print_exc()
              pass
          try:
              self.tableNames[dbAlias].remove(tableName)
          except:
#              traceback.print_exc()
              pass

  def reconnect(self,dbAlias,reloadTables=None):
      self.con.close()
      self.con = self.engine[dbAlias].connect()
      if reloadTables:
         self.dbTables = {}
         self.tableNames = {}
      self.loadTableNamess(dbAlias)

  def close(self,dbAlias):
      self.con.close()
#      for dict in ['engine','dbTables','dbType','metaDict','drivers','tableNames']:
      for dict in self.members:
          try:
              getattr(self,dict).pop(dbAlias)
          except:
              pass
      
  def parse(self,arg):
      """ 
         SQLAlchemy support the following format driver://username:password@host:port/database,
         while here we extend it to the following structure (suitable for ORACLE)
         driver://username:password@host:port/database:dbOwner
      """
      port = None
      host = None
      owner= None
      fileName = None
      dbName = None
      dbUser = None
      dbPass = None
      try:
          driver, dbparams = arg.split("://")
      except:
          msg = "Fail to parse connect argument '%s'\n"%arg
          msg+= cmdDictExt['connect']
          raise msg+"\n"
      if  dbparams.find("@")!=-1:
          user_pass, rest  = dbparams.split("@")
          dbUser,dbPass    = user_pass.split(":")
          if  rest.find("/")!=-1:
              host_port, dbrest= rest.split("/")
              try:
                  host, port = host_port.split(":") 
              except:
                  host = host_port
                  pass
          else:
              dbrest = rest
          try:
              dbName, owner  = dbrest.split(":")
          except:
              dbName = dbrest
              pass
      else: # in case of SQLite, dbparams is a file name
          fileName = dbparams
          dbName   = fileName.split("/")[-1]
          if driver!='sqlite':
             msg = "'%s' parameter is not supported for driver '%s'"%(fileName,driver)
             raise msg+"\n"
#      print "driver",driver
#      print "dbName",dbName
#      print "dbUser",dbUser
#      print "dbPass",dbPass
#      print "host",host
#      print "port",port
#      print "owner",owner
#      print "file",fileName
      return (driver.lower(),dbName,dbUser,dbPass,host,port,owner,fileName)

  def connect(self,driver):
      """Connect to DB"""
      if self.drivers.has_key(driver):
         arg = self.drivers[driver]
      else:
         arg = driver
      dbType,dbName,dbUser,dbPass,host,port,dbOwner,fileName = self.parse(arg)
      dbSchema=None
      if dbType =='oracle' and dbOwner:
         dbAlias = '%s-%s'%(dbName,dbOwner)
         dbSchema= dbOwner.lower()
      else:
         dbAlias = dbName
         if host: dbAlias+="-"+host
      if  not self.drivers.has_key(driver):
          self.drivers[dbAlias]=driver
          self.aliases[driver]=dbAlias
      eType  = string.lower(dbType)
      print "Connecting to %s (%s back-end), please wait ..."%(dbAlias,dbType)

      # Initialize SQLAlchemy engines
      if  not self.engine.has_key(dbAlias):
          eName=""
          if eType=='sqlite':
             eName = "%s:///%s"%(eType,fileName)
             engine= sqlalchemy.create_engine(eName)
          elif eType=='oracle':
             eName = "%s://%s:%s@%s"%(eType,dbUser,dbPass,dbName)
             engine= sqlalchemy.create_engine(eName,strategy='threadlocal',threaded=True)
          elif eType=='mysql':
             eName = "%s://%s:%s@%s/%s"%(eType,dbUser,dbPass,host,dbName)
             engine= sqlalchemy.create_engine(eName,strategy='threadlocal')
          else:
             printExcept("Unsupported DB engine back-end")
          self.engine[dbAlias]=engine
      self.con = self.engine[dbAlias].connect()
      if  not self.dbType.has_key(dbAlias): self.dbType[dbAlias] = eType
      if  not self.dbOwner.has_key(dbAlias): self.dbOwner[dbAlias] = dbOwner
      if  not self.dbSchema.has_key(dbAlias): self.dbSchema[dbAlias] = dbSchema
      if  not self.metaDict.has_key(dbAlias):
          dbMeta = sqlalchemy.MetaData()
          dbMeta.bind=self.engine[dbAlias]
          self.metaDict[dbAlias]=dbMeta
      if  not self.tableNames.has_key(dbAlias): 
          self.tableNames[dbAlias] = self.loadTableNames(dbAlias)
      return self.con

  def loadTableNames(self,dbAlias):
      eType = self.dbType[dbAlias]
      engine = self.engine[dbAlias]
      if  eType=='oracle' and self.dbOwner[dbAlias]:
          query="SELECT table_name FROM all_tables WHERE owner='%s'"%self.dbOwner[dbAlias]
          result=self.con.execute(query)
          tableNames = [x[0].lower().split(".")[-1] for x in result]
      else:
          tableNames=engine.table_names()
      return tableNames

  def loadTables(self,dbAlias,tableName=None):
      if  self.dbTables.has_key(dbAlias):
          tables = self.dbTables[dbAlias]
      else:
          tables = {}
      dbMeta= self.metaDict[dbAlias]
      eType = self.dbType[dbAlias]
      kwargs={'autoload':True}
      for tName in self.tableNames[dbAlias]:
          tabName = tName.lower()
          if tables.has_key(tabName): continue
          if tableName and tabName!=tableName: continue
          if eType=='oracle':
              if  self.dbOwner[dbAlias] and self.dbSchema[dbAlias]:
                  kwargs['schema']=self.dbSchema[dbAlias]
              kwargs['oracle_renums']=True
              kwargs['useexisting']=True
          print "-- Loading '%s' table"%tabName
          tables[tabName]=sqlalchemy.Table(tabName,dbMeta,**kwargs)
      self.dbTables[dbAlias]=tables
      return tables

  def printTable(self,tList,oList,lList,msg=None):
      if self.printMethod=="txt":
         self.printMgr.printTXT(tList,oList,lList,msg)
      elif self.printMethod=="xml":
         self.printMgr.printXML(tList,oList,lList,msg)
      elif self.printMethod=="html":
         self.printMgr.printHTML(tList,oList,lList,msg)
      elif self.printMethod=="csv":
         self.printMgr.printCSV(tList,oList,lList,msg)
      else:
         self.printMgr.printTXT(tList,oList,lList,msg)

  def printList(self,iList,msg=None):
      if  msg:
          s="-"*len(msg)
          print msg
          print s
      for item in iList:
          print item

#
# main
#
if __name__ == "__main__":
   db = DBManager()
   db.connect('sqlite://test.db')
   db.showTable('test.db')
   db.execute('test.db',"select * from test")



#!/usr/bin/env python
#-*- coding: ISO-8859-1 -*-
##########################################################################
#  Copyright (C) 2008 Valentin Kuznetsov <vkuznet@gmail.com>
#  All rights reserved.
#  Distributed under the terms of the BSD License.  The full license is in
#  the file doc/LICENSE, distributed as part of this software.
##########################################################################

cmdDict = {
 "connect":"Invoke connection to DB",
 "close":"Close connection to DB",
 "dbhelp":"dbsh help",
 "rerun":"Re-run command",
 "select":"Execute select SQL statement",
 "insert":"Execute insert SQL statement",
 "drop":"Execute drop SQL statements",
 "update":"Execute update SQL statement",
 "create":"Execute create SQL statement",
 "alter":"Execute aleter SQL statement",
 "show":"Show information about DB, e.g. show tables",
 "desc":"Print table description",
 "page":"Set pagination",
 "migrate":"Migrate content of current DB into another one",
 "mydb":"List all known (connected) DBs",
 "set":"Execute set SQL statement",
 "rollback":"Execute rollback SQL statement",
 "begin":"Execute begin SQL statement",
 "commit":"Execute commit SQL statement",
 "explain":"Execute explain SQL statement",
 "source":"Execute SQL from external file",
 "format":"Set-up formatting output",
 "plot":"Produce plot of favorite SQL",
 "schema":"Schema manipulation command",
 "next":"print next set of results",
 "prev":"print previous set of results",
}
cmdDictExt = {
 "connect":"""
Invoke connection to DB via the following connector

       driver://username:password@host:port/database:owner

The following parameters are optional: host, port, owner. Below you can find all examples.
 MySQL   mysql://user:password@localhost:port/dbname
         mysql://user:password@localhost/dbname,
 ORACLE  oracle://user:password@dns/dbname (use ORACLE TNS service)
         oracle://user:password@127.0.0.1:1521/sidname,
 SQLite  sqlite:////absolute/path/to/database.txt
         sqlite:///relative/path/to/database.txt,
""",
 "close":"Close connection to DB. Once you connected to DB you may wish to close this connection. The close support both syntax via driver or db alias. For full list of known db aliases see mydb command",
 "dbhelp":"dbsh help. It support optional parameter to give you comprehensive help about given keyword, e.g. dbhelp connect will print out extended version of help section for 'connect' keyword.",
 "rerun":"Re-run command corresponding to given number, e.g. rerun 5 will rerun 5th command from history.",
 "select":"""
Execute select statement, dbsh accepts a full SQL syntax appropriate for given DB back-end.
SQL ANSI support the following select statments:

        SELECT expression_list
        FROM data_source
        WHERE predicates
        GROUP BY expression_list
        HAVING predicates
        ORDER BY expression_list

""",
 "insert":"""
Execute insert SQL statement, dbsh accepts a full SQL syntax appropriate for given DB back-end.
SQL ANSI support the following insert statments:

        INSER INTO table_name
        (column_list)
        VALUES
        (value list matched columnt_list)

""",
 "drop":"""
Execute insert SQL statement, dbsh accepts a full SQL syntax appropriate for given DB back-end.
Please use you DB vendor documentation to see if it support DROP statement. But dbsh support at
least two command. One is to drop one table:

        DROP TABLE table_name

And another is to drop all tables in connected database via the following statement:

        DROP DATABASE

""",
 "update":"""
Execute update SQL statement, dbsh accepts a full SQL syntax appropriate for given DB back-end.
SQL ANSI support the following update statments:

        UPDATE TABLE table_name
        SET column_name = value, column_name = value, ...
        WHERE predicates

""",
 "create":"""
Execute create SQL statement, dbsh accepts a full SQL syntax appropriate for given DB back-end.
You can create new table in a database by issuing CREATE TABLE statement. The syntax varies widely
among vendors. Please consult your DB back-end documentation. Here we illustrate a reasonable
example:

        CREATE TABLE table_name (
            id INTEGER,
            name VARCHAR(10),
            timestamp DATE
        );

""",
 "alter":"""
Execute aleter SQL statement, dbsh accepts a full SQL syntax appropriate for given DB back-end.
We support DB back-end syntax which varies among vendors. In general the ALTER command is the
following:

        ALTER TABLE table_name
        ADD COLUMN column_name <column_type>
        ADD CONSTRAIN constrain_name CHECK(condition)
        ..........................
        ALTER COLUMN column_name SET <new data type or assign default, etc.>
        ..........................
        DROP CONSTRAIN constrain_name

""",
 "show":"Show information about DB, e.g. show tables shows all found tables in connected DB",
 "desc":"Prints out table description",
 "page":"Set pagination, support one or two arguments. Example page <offset> <limit>",
 "migrate":"Migrate content of current DB into another one",
 "mydb":"List all known (connected) DBs",
 "set":"""
Execute set SQL statement. Varios DB vendors use SET command for different purposes, mainly to setup transactions. For exact syntax please consult your DB vendor documentation.
""",
 "rollback":"""
Execute rollback SQL statement. In most cases the ROLLBACK statement has been used in transaction content to rollback current transaction. For exact syntax please consult your DB vendor documentation.
""",
 "begin":"""Execute begin SQL statement. In most cases the BEGIN statement indicates start of transaction. For exact syntax please consult your DB vendor documentation.
""",
 "commit":"""Execute commite SQL statement. In most cases the COMMIT statement indicates commitment of current transaction. For exact syntax please consult your DB vendor documentation.
""",
 "explain":"""Execute explain SQL statement. For exact syntax please consult your DB vendor documentation. Here is basic usage:

        EXPLAIN select * from Table;

""",
 "source":"""Execute SQL from external file, e.g. source file.sql, where file.sql contains a set of SQL statements.""",
 "format":"""Set-up formatting output. We support the following formats: txt, html, xml, csv (comma separating values). Once you issue this command the underlyin format will be used for all subsequentive queries. 
""",
 "plot":"""Produce plot of favorite SQL, we support 2D plotting via pylab module. Just use plot command in conjunction with your SQL statement like 

        plot select x,y from table

and x,y will be plotted for you.
""",
 "schema":"""Schema manipulation command, it supports the following options
        schema # without any options will print DDL for your schema
        schema > file.out # will save your DDL in provided file
        schema graph <format> # will produce a graphical graph of your schema, supported via graphviz software
""",
 "next":"print next set of results for executed SQL statement, if page is set-up pagination",
 "prev":"print previous set of results for executed SQL statement, if page is set-up pagination",
}



#!/usr/bin/env python
#-*- coding: ISO-8859-1 -*-
# This code belong to Andrew J. Dolgert <ajd27@cornell.edu> and used by him
# and Valentin Kuznetsov for other project.
#
class DotGraph(object):
    def __init__(self, file):
        self.name = "G"
        self.edges = []
        self.out = file
    
    def Name(self,name):
        self.name = name
        
    def AddEdge(self,start,finish):
        self.edges.append((start,finish))
        
    def Finish(self):
        print >>self.out, "digraph %s {" % (self.name)
        for edge in self.edges:
            print >>self.out, "    %s -> %s;" % (edge)
        print >>self.out, "}"




#!/usr/bin/env python
#-*- coding: ISO-8859-1 -*-
##########################################################################
#  Copyright (C) 2008 Valentin Kuznetsov <vkuznet@gmail.com>
#  All rights reserved.
#  Distributed under the terms of the BSD License.  The full license is in
#  the file doc/LICENSE, distributed as part of this software.
##########################################################################

# system modules
import os, sys, time, types, string, popen2, traceback

# ipython modules
from IPython import Release
import IPython.ipapi
ip = IPython.ipapi.get()

import dbsh
from dbsh.dbshell import *
from dbsh.dbcmd import *
from dbsh.dbprint import *
    
import __main__ 


try:
    db=DBManager()
    dbprint=PrintOutput()
except:
    traceback.print_exc()
    raise "ERROR: fail to load DBManager"

__alias_table__={}
def is_ready():
    if  not db.con:
        print "This function is not available until you connect to DB"
        return 1
    return 0

def setPrompt(in1):
    IP = __main__.__dict__['__IP'] 
    prompt = getattr(IP.outputcache,'prompt1') 
    prompt.p_template = in1
    prompt.set_p_str() 

def getPrompt():
    IP = __main__.__dict__['__IP'] 
    prompt = getattr(IP.outputcache,'prompt1') 
    return IP.outputcache.prompt1.p_template

def dbAlias():
    prompt = getPrompt()
#    return prompt.split("|")[0].split("-")[1].strip() # see how dbshell define dbname and dbAlias
    p = prompt.split("|")[0].split("-")[1:]
    alias = '-'.join(p).strip()
    return alias # see how dbshell define dbname and dbAlias

def connect(self,arg):
    db.connect(arg)
    setPrompt(db.dbname(arg))

def close(self,arg):
    if is_ready(): return
    db.close(dbAlias())
    setPrompt('dbsh |\#> ')

def rerun(self,arg):
    ip.ex("exec _i%d" % int(arg))

def show(self,arg):
    if is_ready(): return
    if arg=='tables':
       db.showTable(dbAlias())
    else:
       db.execute(dbAlias(),"show "+arg.replace(";",""))
#       print dbprint.msgRED("ERROR: '%s' not supported keyword"%arg)

def desc(self,arg):
    if is_ready(): return
    db.desc(dbAlias(),arg.replace(";",""))

def page(self,arg):
    db.page(arg)

def next(self,arg):
    db.next()

def prev(self,arg):
    db.prev()

def migrate(self,arg):
    if is_ready(): return
    db.migrate(dbAlias(),arg)

def mydb(self,arg):
    if len(db.dbType.keys()):
        _msg="You are connected to the following DBs:"
        msg =_msg+"\n"+"-"*len(_msg)
        print msg
        for key in db.dbType.keys():
            print "%s (%s back-end)"%(key,db.dbType[key])    
    else:
        print "No connection found."

def alias(self,arg):
    name,fstring = arg.split(" ",1)
    print "new alias: %s <%s>"%(dbprint.msgGREEN(name),fstring)
    __alias_table__[name]=fstring
    func,params = fstring.split(" ",1)
    def afunc(self,arg):
        print fstring
        ip.magic("%%%s"%fstring)
    ip.expose_magic(name,afunc)

def dbhelp(self,arg):
    cmdList = cmdDict.keys()
    cmdList.sort()
    sep = 0
    for i in cmdList:
        if sep<len(str(i)): sep=len(str(i))
    if  not arg:
        msg = "Available commands:\n"
        for cmd in cmdList:
            msg+="%s%s %s\n"%(dbprint.msgGREEN(cmd)," "*abs(sep-len(cmd)),cmdDict[cmd])
    else:
        cmd = arg.strip()
        if cmdDictExt.has_key(cmd):
           msg = "\n%s: %s\n"%(dbprint.msgGREEN(cmd),cmdDictExt[cmd])
        else:
           msg = dbprint.msgRED("\nSuch command is not available\n")
    print msg

def source(self,arg):
    if is_ready(): return
    f = None
    try:
        f = open(arg,'r')
    except:
        msg = "No such file '%s'"%arg
        raise msg
    qList = f.read().split(';')
    for q in qList:
        query = f.read().replace('\n',' ').replace(";","")
        print "\n+++EXECUTE:",query
        db.execute(dbAlias(),query)

def format(self,arg):
    if is_ready(): return
    if not (arg.lower()=="txt" or arg.lower()=="xml" or arg.lower()=="html" or arg.lower()=="csv"):
       raise dbprint.msgRED("ERROR: not supported format, please use txt,xml,html,csv")
    db.printMethod=arg
    
def plot(self,arg):
    if is_ready(): return
    db.plot(dbAlias(),arg)
    
def schema(self,arg):
    if is_ready(): return
    aList = arg.split()
    if  not arg:
        db.dump(dbAlias(),arg)
    elif aList[0]=="graph":
        format = None
        if len(aList)>1: format = aList[1]
        db.writeGraph(dbAlias(),format)
    elif aList[0]==">":
        db.dump(dbAlias(),aList[1])

#
# Immitate SQL statements
#
def select(self,arg):
    if arg and arg.lower()=='help': return dbhelp(self,'select')
    if is_ready(): return
    db.execute(dbAlias(),"select "+arg.replace(";",""))

def insert(self,arg):
    if arg and arg.lower()=='help': return dbhelp(self,'insert')
    if is_ready(): return
    db.execute(dbAlias(),"insert "+arg.replace(";",""),listResults=0)

def drop(self,arg):
    if arg and arg.lower()=='help': return dbhelp(self,'drop')
    if is_ready(): return
    params = arg.replace(";","").split()
    if params[0].lower()=="table":
       db.dropTable(dbAlias(),params[1])
    elif params[0].lower()=="database":
       db.dropDB(dbAlias())
    else:
       db.execute(dbAlias(),"drop "+arg.replace(";",""),listResults=0)
       db.reconnect(dbAlias(),reloadTables=1)
    msg = "'%s' executed successfully"%("drop "+arg,)
    print dbprint.msgGREEN(msg)

def update(self,arg):
    if arg and arg.lower()=='help': return dbhelp(self,'update')
    if is_ready(): return
    db.execute(dbAlias(),"update "+arg.replace(";",""),listResults=0)

def create(self,arg):
    if arg and arg.lower()=='help': return dbhelp(self,'create')
    if is_ready(): return
    db.execute(dbAlias(),"create "+arg.replace(";",""),listResults=0)
    db.reconnect(dbAlias())
    msg = "'%s' executed successfully"%("create "+arg,)
    print dbprint.msgGREEN(msg)

def alter(self,arg):
    if arg and arg.lower()=='help': return dbhelp(self,'alter')
    if is_ready(): return
    db.execute(dbAlias(),"alter "+arg.replace(";",""),listResults=0)

def set(self,arg):
    if arg and arg.lower()=='help': return dbhelp(self,'set')
    if is_ready(): return
    db.execute(dbAlias(),"set "+arg.replace(";",""),listResults=0)

def rollback(self,arg):
    if arg and arg.lower()=='help': return dbhelp(self,'rollback')
    if is_ready(): return
    db.execute(dbAlias(),"rollback "+arg.replace(";",""),listResults=0)

def begin(self,arg):
    if arg and arg.lower()=='help': return dbhelp(self,'begin')
    if is_ready(): return
    db.execute(dbAlias(),"begin "+arg.replace(";",""),listResults=0)

def commit(self,arg):
    if arg and arg.lower()=='help': return dbhelp(self,'commit')
    if is_ready(): return
    db.execute(dbAlias(),"commit "+arg.replace(";",""),listResults=0)

def explain(self,arg):
    if arg and arg.lower()=='help': return dbhelp(self,'explain')
    if is_ready(): return
    db.execute(dbAlias(),"explain "+arg.replace(";",""))

#
# Main function
#
def main():
    o = ip.options
    ip.expose_magic('connect',connect)

    # SQL commands
    ip.expose_magic('select',select)
    ip.expose_magic('SELECT',select)
    ip.expose_magic('insert',insert)
    ip.expose_magic('INSERT',insert)
    ip.expose_magic('drop',drop)
    ip.expose_magic('DROP',drop)
    ip.expose_magic('update',update)
    ip.expose_magic('UPDATE',update)
    ip.expose_magic('create',create)
    ip.expose_magic('CREATE',create)
    ip.expose_magic('alter',alter)
    ip.expose_magic('ALTER',alter)
    ip.expose_magic('set',set)
    ip.expose_magic('SET',set)
    ip.expose_magic('rollback',rollback)
    ip.expose_magic('ROLLBACK',rollback)
    ip.expose_magic('begin',begin)
    ip.expose_magic('BEGIN',begin)
    ip.expose_magic('commit',commit)
    ip.expose_magic('COMMIT',commit)
    ip.expose_magic('explain',explain)
    ip.expose_magic('EXPLAIN',explain)

    ip.expose_magic('close',close)
    ip.expose_magic('rerun',rerun)
    ip.expose_magic('dbhelp',dbhelp)
    ip.expose_magic('show',show)
    ip.expose_magic('desc',desc)
    ip.expose_magic('page',page)
    ip.expose_magic('next',next)
    ip.expose_magic('prev',prev)
    ip.expose_magic('migrate',migrate)
    ip.expose_magic('mydb',mydb)
    ip.expose_magic('alias',alias)
    ip.expose_magic('source',source)
    ip.expose_magic('format',format)
    ip.expose_magic('plot',plot)
    ip.expose_magic('schema',schema)

    # autocall to "full" mode (smart mode is default, I like full mode)
    o.autocall = 2
    
    # Jason Orendorff's path class is handy to have in user namespace
    # if you are doing shell-like stuff
    try:
        ip.ex("from path import path" )
    except ImportError:
        pass
    
    ip.ex('import os')
        
    # Set dbsh prompt
    o.prompt_in1= 'dbsh |\#> '
    o.prompt_in2= 'dbsh> '
    o.prompt_out= '<\#> '
    
    # define dbsh banner
    ver = "%s.%s"%(dbsh.__version__,dbsh.__revision__)
    pyver = sys.version.split('\n')[0]
    ipyver = Release.version
    msg = "Welcome to dbsh %s!\n[python %s, ipython %s]\n%s\n"%(ver,pyver,ipyver,os.uname()[3])
    msg+="For dbsh help use "+dbprint.msgBLUE("dbhelp")+", for python help use help commands\n"
    o.banner = msg
    o.prompts_pad_left="1"
    # Remove all blank lines in between prompts, like a normal shell.
    o.separate_in="0"
    o.separate_out="0"
    o.separate_out2="0"
    
main()



#!/usr/bin/env python
#-*- coding: ISO-8859-1 -*-
##########################################################################
#  Copyright (C) 2008 Valentin Kuznetsov <vkuznet@gmail.com>
#  All rights reserved.
#  Distributed under the terms of the BSD License.  The full license is in
#  the file doc/LICENSE, distributed as part of this software.
##########################################################################
"""
dbsh -- an interactive shell for DB back-end.

Every known DB has its own shell for SQL operations.
The dbshell (dbsh) provides unique, interactive, programmable environment
and simple shell to connect and manipulate your favorite DB contexts. 
It supports naturally a syntax of DB you use to, for example, while you connect to
MySQL dbsh supports full syntax of MySQL, but when you connect to ORACLE it supports
ORALCE SQL and commands.

It is based on two components:
- interactive python IPython, http://ipython.scipy.org/moin/About.
- SQLAlchemy, http://www.sqlalchemy.org

It uses SQLAlchemy ORB tool as an SQL layer to perform interactive operations against given DB-backend.
Therefore it provides common shell for ANY DB back-end in uniform fashion.
All SQL operations supported by given DB back-end naturally supported by dbsh.

$Id: __init__.py,v 1.7 2008/09/04 19:02:25 vkuznet Exp $
"""

# Define what gets imported with a 'from dbsh import *'
#__all__ = ['dbshell','dbcmd','dbprint','ipy_profile_dbsh.py','utils.py']

__author__   = "Valentin Kuznetsov <vkuznet@gmail.com>"
__license__  = "BSD"
__revision__ = 6
__version__  = 0.1





#!/usr/bin/env python
#-*- coding: ISO-8859-1 -*-
##########################################################################
#  Copyright (C) 2008 Valentin Kuznetsov <vkuznet@gmail.com>
#  All rights reserved.
#  Distributed under the terms of the BSD License.  The full license is in
#  the file doc/LICENSE, distributed as part of this software.
##########################################################################
"""
Common utilities
"""
from dbsh import *

# Python Cookbook
def fetchsome(cursor, arraysize=10):
    """A generator that simplifies the use of fetchmany"""
    while True:
          results = cursor.fetchmany(arraysize)
          if not results: break
          for result in results:
              yield result
def fetchFromTo(cursor, iLimit=10, iOffset=0):
    """A way to retrieve a limited number of rows"""
#    if iOffset:
#       for i in xrange(0,iOffset):
#           cursor.next()
    results = cursor.fetchmany(iLimit)
    if not results: return
    for result in results:
        yield result

def swapDict(original_dict):
    return dict([(v, k) for (k, v) in original_dict.iteritems()])

def elementwise(fn):
    def newfn(arg):
        if hasattr(arg,'__getitem__'):  # is a Sequence
            return type(arg)(map(fn, arg))
        else:
            return fn(arg)
    return newfn

@elementwise
def itemLen(x):
    return len(str(x))

def makeTIME(intime):
    return time.strftime("%a, %d %b %Y %H:%M:%S GMT",time.gmtime(long(intime)))




""" User configuration file for IPython

This is a more flexible and safe way to configure ipython than *rc files
(ipythonrc, ipythonrc-pysh etc.)

This file is always imported on ipython startup. You can import the
ipython extensions you need here (see IPython/Extensions directory).

Feel free to edit this file to customize your ipython experience.

Note that as such this file does nothing, for backwards compatibility.
Consult e.g. file 'ipy_profile_sh.py' for an example of the things 
you can do here.

See http://ipython.scipy.org/moin/IpythonExtensionApi for detailed
description on what you could do here.
"""

# Most of your config files and extensions will probably start with this import

import IPython.ipapi
ip = IPython.ipapi.get()

# You probably want to uncomment this if you did %upgrade -nolegacy
# import ipy_defaults    

import os   

def main():   

    # uncomment if you want to get ipython -p sh behaviour
    # without having to use command line switches  
    # import ipy_profile_sh

    # Configure your favourite editor?
    # Good idea e.g. for %edit os.path.isfile

    #import ipy_editors
    
    # Choose one of these:
    
    #ipy_editors.scite()
    #ipy_editors.scite('c:/opt/scite/scite.exe')
    #ipy_editors.komodo()
    #ipy_editors.idle()
    # ... or many others, try 'ipy_editors??' after import to see them
    
    # Or roll your own:
    #ipy_editors.install_editor("c:/opt/jed +$line $file")
    
    
    o = ip.options
    # An example on how to set options
    #o.autocall = 1
    o.system_verbose = 0
    
    #import_all("os sys")
    #execf('~/_ipython/ns.py')


    # -- prompt
    # A different, more compact set of prompts from the default ones, that
    # always show your current location in the filesystem:

    #o.prompt_in1 = r'\C_LightBlue[\C_LightCyan\Y2\C_LightBlue]\C_Normal\n\C_Green|\#>'
    #o.prompt_in2 = r'.\D: '
    #o.prompt_out = r'[\#] '
    
    # Try one of these color settings if you can't read the text easily
    # autoexec is a list of IPython commands to execute on startup
    #o.autoexec.append('%colors LightBG')
    #o.autoexec.append('%colors NoColor')
    #o.autoexec.append('%colors Linux')
    
    # for sane integer division that converts to float (1/2 == 0.5)
    #o.autoexec.append('from __future__ import division')
    
    # For %tasks and %kill
    #import jobctrl 
    
    # For autoreloading of modules (%autoreload, %aimport)    
    #import ipy_autoreload
    
    # For winpdb support (%wdb)
    #import ipy_winpdb
    
    # For bzr completer, requires bzrlib (the python installation of bzr)
    #ip.load('ipy_bzr')
    
    # Tab completer that is not quite so picky (i.e. 
    # "foo".<TAB> and str(2).<TAB> will work). Complete 
    # at your own risk!
    #import ipy_greedycompleter
    
    
    
# some config helper functions you can use 
def import_all(modules):
    """ Usage: import_all("os sys") """ 
    for m in modules.split():
        ip.ex("from %s import *" % m)
        
def execf(fname):
    """ Execute a file in user namespace """
    ip.ex('execfile("%s")' % os.path.expanduser(fname))

main()



#!/usr/bin/env python

import os, sys
import ez_setup
ez_setup.use_setuptools()

from setuptools import setup, find_packages
from sys import version
from src import dbsh

if sys.version_info < (2, 4):
    raise Exception("dbsh requires Python 2.4 or higher.")

scriptfiles=filter(os.path.isfile, ['bin/dbsh'])
setup(
    name = "dbsh",
    version = "%s.%s"%(dbsh.__version__,dbsh.__revision__),
    description = "Common, interactive, programmable environment and simple shell to connect and manipulate your favorite DB content",
    package_dir = {'dbsh':'src/dbsh'},
    data_files = [('config',['src/config/ipythonrc'])],
    packages = find_packages('src'),
    include_package_data = True,
    zip_safe = False,
    scripts = scriptfiles,
    long_description = """\

dbsh -- an interactive shell for DB back-end.

Every known DB has its own shell for SQL operations.
The dbshell (dbsh) provides unique, interactive, programmable environment
and simple shell to connect and manipulate your favorite DB contexts. 
It supports naturally a syntax of DB you use to, for example, while you connect to
MySQL dbsh supports full syntax of MySQL, but when you connect to ORACLE it supports
ORALCE SQL and commands.

It is based on two components:
- interactive python IPython, http://ipython.scipy.org/moin/About.
- SQLAlchemy, http://www.sqlalchemy.org

It uses SQLAlchemy ORB tool as an SQL layer to perform interactive operations against given DB-backend.
Therefore it provides common shell for ANY DB back-end in uniform fashion.
All SQL operations supported by given DB back-end naturally supported by dbsh.

""",
    classifiers = [
        'Environment :: Console',
        "Intended Audience :: Developers",
        'Intended Audience :: End Users/Desktop',
        'Intended Audience :: System Administrators',
        "License :: OSI Approved :: BSD License",
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: POSIX',
        "Programming Language :: Python",
        "Topic :: Database :: Front-Ends",
    ],

    dependency_links = ['http://www.sqlalchemy.org/','http://ipython.scipy.org/moin/About'],
    install_requires=["IPython", "SQLAlchemy"],
#    dependency_links = ['http://www.sqlalchemy.org/','http://ipython.scipy.org/moin/About','http://numpy.scipy.org/','http://matplotlib.sourceforge.net/'],
#    install_requires=["IPython", "SQLAlchemy","numpy","matplotlib"],
    author = "Valentin Kuznetsov",
    author_email = "dbsh4u@gmail.com",
    url = "http://www.dbshell.org",
    license = "BSD License",
)

